#define MOZ_SOURCE_STAMP 6a7c1c8db5548d077c7fa36bce41af629ba52bd8
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/6a7c1c8db5548d077c7fa36bce41af629ba52bd8
